<?php

function phpbb_sso_admin() {
    $form = array();

    $form['phpbb'] = array(
        '#type' => 'fieldset',
        '#title' => t('phpBB Settings'),
        '#collapsible' => true,
        '#collapsed' => false,
    );

    $form['phpbb']['db'] = array(
        '#type' => 'fieldset',
        '#title' => t('Database'),
        '#collapsible' => false,
        '#collapsed' => false,
    );

    $form['phpbb']['db']['phpbb_sso_phpbb_db_name'] = array(
        '#type' => 'textfield',
        '#title' => t('phpBB database'),
        '#description' => t('The database name of your phpBB install. Leave blank to use the current Drupal database.'),
        '#default_value' => variable_get('phpbb_sso_phpbb_db_name', ''),
    );

    $form['phpbb']['db']['phpbb_sso_phpbb_table_prefix'] = array(
        '#type' => 'textfield',
        '#title' => t('phpBB table prefix'),
        '#description' => t('The table prefix you chose when installing phpBB.')
                        . '<br/>' . t('Eg. Use <em>phpbb</em> if all your phpBB tables begin with <em>phpbb_</em>.'),
        '#default_value' => variable_get('phpbb_sso_phpbb_table_prefix', ''),
        '#required' => true,
    );

    $form['phpbb']['db']['advanced'] = array(
        '#type' => 'fieldset',
        '#title' => t('Advanced options'),
        '#collapsible' => true,
        '#collapsed' => true,
    );

    global $db_url;
    $url = parse_url($db_url);

    require_once 'includes/install.inc';
    $db_types = drupal_detect_database_types();

    if (count($db_types) > 1) {
      $form['phpbb']['db']['advanced']['phpbb_sso_phpbb_db_type'] = array(
          '#type' => 'select',
          '#title' => t('Database type'),
          '#options' => $db_types,
          '#description' => t('The type of database your phpBB data has been stored in.'),
          '#default_value' => variable_get('phpbb_sso_phpbb_db_type', $url['scheme']),
          '#maxlength' => 255,
      );
    }

    $form['phpbb']['db']['advanced']['phpbb_sso_phpbb_db_host'] = array(
        '#type' => 'textfield',
        '#title' => t('Database host'),
        '#description' => t('The database server that contains your phpBB install.'),
        '#default_value' => variable_get('phpbb_sso_phpbb_db_host', $url['host']),
        '#size' => 45,
        '#maxlength' => 255,
        '#description' => t('If your phpBB database is located on a different server, change this.'),
    );

    $form['phpbb']['db']['advanced']['phpbb_sso_phpbb_db_port'] = array(
        '#type' => 'textfield',
        '#title' => t('Database port'),
        '#default_value' => variable_get('phpbb_sso_phpbb_db_port', $url['port']),
        '#size' => 45,
        // The maximum port number is 65536, 5 digits.
        '#maxlength' => 5,
        '#description' => t('If your phpBB database server is listening to a non-standard port, enter its number.'),
    );

    $form['phpbb']['db']['advanced']['phpbb_sso_phpbb_db_username'] = array(
        '#type' => 'textfield',
        '#title' => t('Database username'),
        '#description' => t('The databse username used to access your phpBB install. Leave blank to use the current Drupal database username.'),
        '#default_value' => variable_get('phpbb_sso_phpbb_db_username', $url['user']),
    );

    $form['phpbb']['db']['advanced']['phpbb_sso_phpbb_db_password'] = array(
        '#type' => 'password',
        '#title' => t('Database password'),
        '#description' => t('The database password used to access your phpBB install. Leave blank to use the current Drupal database password.'),
    );

    $form['phpbb']['phpbb_sso_phpbb_path'] = array(
        '#type' => 'textfield',
        '#title' => t('phpBB path'),
        '#description' => t('The directory path to your phpBB install, relative to your Drupal install.')
                        . '<br/>Eg. Use <em>phpbb3</em> if phpBB was installed into a directory named <em>phpbb3</em> and resides in your Drupal root directory.',
        '#default_value' => variable_get('phpbb_sso_phpbb_path', ''),
        '#required' => true,
    );

    $form['phpbb']['header'] = array(
        '#markup' => '<p><h3>Please note</h3>
<ol>
<li>If your Drupal website is located at <em>example.com</em>, phpBB must be located either at <em>example.com</em> or <em>subdomain.example.com</em>, but not <em>subdomain1.subdomain2.example.com</em>, or <em>other-website.com</em></li>
<li>Your Drupal website domain name <em>and</em> phpBB website domain name must contain a dot (.). For example, <em>localhost.localdomain</em> will work, but <em>localhost</em> would not work.</li>
</p>',
    );

    $form['debug'] = array(
        '#type' => 'fieldset',
        '#title' => t('Debugging'),
        '#collapsible' => true,
        '#collapsed' => true,
    );

    $form['debug']['phpbb_sso_debug_enabled'] = array(
        '#type' => 'checkbox',
        '#title' => t('Enable debugging'),
        '#description' => t('Enable or disable all debugging information.'),
        '#default_value' => variable_get('phpbb_sso_debug_enabled', false),
    );

    $form['debug']['phpbb_sso_debug_screen'] = array(
        '#type' => 'checkbox',
        '#title' => t('Output: screen'),
        '#description' => t('Print to screen.'),
        '#default_value' => variable_get('phpbb_sso_debug_screen', false),
        '#states' => array(
          'invisible' => array(
            ':input[name="phpbb_sso_debug_enabled"]' => array('checked' => FALSE),
          ),
        ),
    );

    $form['debug']['phpbb_sso_debug_watchdog'] = array(
        '#type' => 'checkbox',
        '#title' => t('Output: watchdog'),
        '#description' => t('Send to watchdog.'),
        '#default_value' => variable_get('phpbb_sso_debug_watchdog', false),
        '#states' => array(
          'invisible' => array(
            ':input[name="phpbb_sso_debug_enabled"]' => array('checked' => FALSE),
          ),
        ),
    );

    $form['debug']['phpbb_sso_debug_devel'] = array(
        '#type' => 'checkbox',
        '#title' => t('Output: devel'),
        '#description' => t('Use the devel module.'),
        '#default_value' => variable_get('phpbb_sso_debug_devel', false),
        '#states' => array(
          'invisible' => array(
            ':input[name="phpbb_sso_debug_enabled"]' => array('checked' => FALSE),
          ),
        ),
    );

    return system_settings_form($form);
}

function phpbb_sso_admin_validate(&$form, &$form_state) {
    global $db_url;
    $url = parse_url($db_url);
    $db_type = empty($form_state['values']['phpbb_sso_phpbb_db_type']) ? $url['scheme'] : $form_state['values']['phpbb_sso_phpbb_db_type'];
    $db_host = empty($form_state['values']['phpbb_sso_phpbb_db_host']) ? $url['host'] : $form_state['values']['phpbb_sso_phpbb_db_host'];
    $db_name = empty($form_state['values']['phpbb_sso_phpbb_db_name']) ? substr($url['path'], 1) : $form_state['values']['phpbb_sso_phpbb_db_name'];
    $db_user = empty($form_state['values']['phpbb_sso_phpbb_db_username']) ? $url['user'] : $form_state['values']['phpbb_sso_phpbb_db_username'];
    $db_pass = empty($form_state['values']['phpbb_sso_phpbb_db_password']) ? variable_get('phpbb_sso_phpbb_db_password', $url['pass']) : $form_state['values']['phpbb_sso_phpbb_db_password'];

    $db_url = $db_type .'://'. urlencode($db_user) . ($db_pass ? ':'. urlencode($db_pass) : '') .'@'. ($db_host ? urlencode($db_host) : 'localhost') . ($db_port ? ":$db_port" : '') .'/'. urlencode($db_name);
    if (!(phpbb_sso_admin_validate_db($db_url)))
        return;

    $table_prefix = $form_state['values']['phpbb_sso_phpbb_table_prefix'];
    if (!empty($table_prefix))
        $table_prefix .= '_';

    $known_table_name = 'users';
    if (!(phpbb_sso_admin_validate_table($db_url, $table_prefix . $known_table_name))) {
        return;
    }

    $path = $form_state['values']['phpbb_sso_phpbb_path'];
    if (!(phpbb_sso_admin_validate_path($path))) {
        return;
    }
}

function phpbb_sso_admin_validate_path($path) {
    if (!is_dir($path)) {
        form_set_error('phpbb_sso_phpbb_path', 'This path does not exist.');
        return false;
    }

    return true;
}

function phpbb_sso_admin_validate_table($db_url, $table) {
    $result = false;
    try {
        phpbb_sso_db_set_active($db_url);

        if (db_table_exists($table)) {
            $result = true;
        } else {
            form_set_error('phpbb_sso_phpbb_table_prefix', 'No phpBB tables could be found using this table prefix.');
        }
    }
    catch (Exception $e) {
        form_set_error('phpbb_sso_phpbb_db_name', 'Database error: ' . print_r($e->getMessage(), true));
    }

    db_set_active('default');
    return $result;
}

function phpbb_sso_admin_validate_db($db_url) {
    if (empty($db_url))
        return true;

    $result = false;
    try {
      $url = parse_url($db_url);
      $function = 'drupal_test_'. $url['scheme'];

      if ($function($db_url, $success))
        $result = true;
      else {
        $error_messages = drupal_get_messages('error', true);
        foreach ($error_messages['error'] as $message) {
          if (preg_match('/(Failed to connect to your MySQL database server..+)<ul><li>Are you sure you have the correct username and password?.+$/', $message, $matches)) {
            if (preg_match('/Unknown MySQL server host/', $matches[1])) {
              form_set_error('phpbb_sso_phpbb_db_host', $matches[1]);
            } else if (preg_match('/Access denied for user/', $matches[1])) {
              form_set_error('phpbb_sso_phpbb_db_username', $matches[1]);
              form_set_error('phpbb_sso_phpbb_db_password', ' ');
            } else {
              form_set_error('', $matches[1]);
            }
          } else {
            drupal_set_message($message, 'error');
          }
        }
      }
    }
    catch (Exception $e) {
        if (preg_match('/Unknown database/i', $e->getMessage())) {
            form_set_error('phpbb_sso_phpbb_db_name', 'This database may not exist, or you do not have sufficient permissions to access the database.');
        } else {
            form_set_error('phpbb_sso_phpbb_db_name', 'Database error: %error.', array('%error' => print_r($e->getMessage(), true)));
        }
    }

    return $result;
}
